@@ -216,7 +216,9 @@ def add_spider_args(parser):
         help="Adds statements to help find why a tool isn't appearing in the toolbox",
     )
     parser.add_argument(
-        "--all", action="store_true", help="Print all tools in alphabetical order",
+        "--all",
+        action="store_true",
+        help="Print all tools in alphabetical order",
     )
 
 
@@ -508,6 +510,12 @@ def add_run_args(parser, add_workflow_argument=True):
         help="maximum GB of memory to use when generating resource overrides",
     )
 
+    inpmanip_args.add_argument(
+        "--max-duration",
+        type=int,
+        help="maximum seconds that a task can request when generating resource overrides",
+    )
+
     # add hints
     hint_args = parser.add_argument_group("hints")
     for HintType in HINTS:
@@ -900,6 +908,7 @@ def do_run(args):
         watch=args.progress,
         max_cores=args.max_cores,
         max_mem=args.max_memory,
+        max_duration=args.max_duration,
         force=args.no_cache,
         recipes=args.recipe,
         keep_intermediate_files=keep_intermediate_files,
@@ -935,7 +944,9 @@ def do_spider(args):
 def do_inputs(args):
 
     if args.config or args.recipe:
-        JanisConfiguration.initial_configuration(path=args.config,)
+        JanisConfiguration.initial_configuration(
+            path=args.config,
+        )
 
     quality_type = None
 
