@@ -1,6 +1,8 @@
+from sys import getsizeof
 from sqlite3 import OperationalError
-from typing import List, TypeVar, Iterable, Union, Callable, Dict, Optional
+from typing import List, TypeVar, Iterable, Union, Callable, Dict, Optional, Tuple
 
+from janis_assistant.data.enums import TaskStatus
 from janis_core import Logger
 
 from janis_assistant.data.dbproviderbase import DbProviderBase
@@ -12,7 +14,7 @@ T = TypeVar("T")
 
 def groupby(
     iterable: Iterable[T], selector: Union[str, Callable[[T], str]]
-) -> Dict[str, T]:
+) -> Dict[str, Iterable[T]]:
     q = {}
     if isinstance(selector, str):
         k = selector
@@ -29,15 +31,24 @@ def groupby(
 
 
 class JobDbProvider(DbProviderBase):
-    def __init__(self, db, submission_id):
+    job_cache_warnings = [10, 100, 500, 1000]
+
+    def __init__(self, db, readonly, submission_id):
         super().__init__(
             base_type=RunJobModel,
             db=db,
+            readonly=readonly,
             tablename="jobs",
             scopes={"submission_id": submission_id},
         )
         self.submission_id = submission_id
-        # self.eventsDB = JobEventDbProvider(self.db, self.wid)
+        self.job_cache_last_idx = 0
+
+        """
+        We seem to persistently have problems with updating the DB, we can't use
+        newer versions of SQLite because of a stakeholder issue.
+        """
+        self._cache_completed_ids: Dict[Tuple, str] = {}
 
     def get_single(
         self, id_: str, run_id: str = RunModel.DEFAULT_ID
@@ -86,27 +97,6 @@ class JobDbProvider(DbProviderBase):
 
         return rets
 
-        # query = "SELECT * FROM jobs WHERE wid = ?"
-        # with self.with_cursor() as cursor:
-        #     try:
-        #         cursor.execute(query, (self.wid,))
-        #         rows = cursor.fetchall()
-        #     except OperationalError as e:
-        #         if "readonly database" in str(e):
-        #             # mfranklin: idk, this sometimes happens. We're doing a select query, idk sqlite3 driver...
-        #             Logger.debug(
-        #                 "Got readonly error when running query: '{query}', skipping for now"
-        #             )
-        #             return None
-        #         elif "locked" in str(e):
-        #             Logger.debug(
-        #                 "We hit the database at the same time the janis process wrote to it, meh"
-        #             )
-        #             return None
-        #         raise e
-        #
-        # return [RunJobModel.from_row(row) for row in rows]
-
     def get_all_mapped(self) -> Optional[List[RunJobModel]]:
 
         alljobs = self.get_all()
@@ -124,6 +114,76 @@ class JobDbProvider(DbProviderBase):
 
         return [j for j in alljobs if j.parent is None]
 
+    ## FILTERING
+
+    def populate_cache_if_required(self):
+        if self._cache_completed_ids is not None:
+            return False
+        self.populate_cache()
+        return True
+
+    def populate_cache(self):
+        self._cache_completed_ids = {}
+        idkeys = set(self.get_id_keys())
+        idkeys_ordered = list(idkeys)
+        prows = f"SELECT {', '.join(idkeys_ordered)}, status FROM {self._tablename}"
+        with self.with_cursor() as cursor:
+            rows = cursor.execute(prows).fetchall()
+
+            if len(rows) == 0:
+                return
+
+            for row in rows:
+                idkeys = tuple(row[:-2])
+                # is_updateable = not TaskStatus(row[-1]).is_in_final_state()
+                self._cache_completed_ids[idkeys] = row[-1]
+
+    def filter_updates(
+        self, jobs: List[RunJobModel], add_inserts_to_cache=True
+    ) -> Tuple[List[RunJobModel], List[RunJobModel]]:
+        # don't call super, it'll break because of the cache
+
+        updates = []
+        inserts = []
+
+        if len(jobs) == 0:
+            return updates, inserts
+
+        self.populate_cache_if_required()
+
+        idkeys = set(self.get_id_keys())
+        idkeys_ordered = list(idkeys)
+        dbalias_map = {t.dbalias: t.name for t in self._base.keymap()}
+        skipped = 0
+
+        for job in jobs:
+            el_idkey = tuple([getattr(job, dbalias_map[_k]) for _k in idkeys_ordered])
+
+            jstatus = self._cache_completed_ids.get(el_idkey)
+            if jstatus is None:
+                inserts.append(job)
+            elif job.status.value != jstatus:
+                updates.append(job)
+            elif jstatus:
+                skipped += 1
+            self._cache_completed_ids[el_idkey] = job.status.value
+
+        if skipped:
+            Logger.log(
+                f"Skipped updating {skipped} jobs as those jobs were already in a final state"
+            )
+        memory = getsizeof(self._cache_completed_ids) // 1024
+        if (
+            self.job_cache_last_idx < len(self.job_cache_warnings)
+            and memory > self.job_cache_warnings[self.job_cache_last_idx]
+        ):
+            Logger.warn(f"Job cache is using {memory} MB")
+            self.job_cache_last_idx += 1
+
+        return updates, inserts
+
+        # We'll remove all updates
+
     def upgrade_schema(self, from_version: int):
         # if from_version < 2:
         #     self.migrate_to_2()
