@@ -10,10 +10,11 @@ from janis_assistant.data.dbproviderbase import DbProviderBase
 
 
 class RunDbProvider(DbProviderBase[RunModel]):
-    def __init__(self, db, submission_id: str):
+    def __init__(self, db, readonly, submission_id: str):
         super().__init__(
             base_type=RunModel,
             db=db,
+            readonly=readonly,
             tablename="runs",
             scopes={"submission_id": submission_id},
         )
@@ -25,10 +26,11 @@ class RunDbProvider(DbProviderBase[RunModel]):
 
 
 class RunStatusDbProvider(DbProviderBase[RunStatusUpdate]):
-    def __init__(self, db, submission_id: str):
+    def __init__(self, db, readonly, submission_id: str):
         super().__init__(
             base_type=RunStatusUpdate,
             db=db,
+            readonly=readonly,
             tablename="runstatus",
             scopes={"submission_id": submission_id},
         )
