@@ -1,4 +1,5 @@
 import json
+import math
 import os
 import re
 import shutil
@@ -6,20 +7,17 @@ import signal
 import subprocess
 import sys
 import threading
-import math
 from datetime import datetime
-from urllib import request, parse
-
 from glob import glob
 from typing import Optional, List, Tuple, Union
+from urllib import request, parse
 
-from janis_assistant.data.models.run import RunModel
-from janis_core import LogLevel
 from janis_core.utils import first_value
 from janis_core.utils.logger import Logger
 
-from janis_assistant.__meta__ import ISSUE_URL
+from janis_assistant.__meta__ import ISSUE_URL, GITHUB_URL
 from janis_assistant.data.models.outputs import WorkflowOutputModel
+from janis_assistant.data.models.run import RunModel
 from janis_assistant.engines.cromwell.cromwellmetadata import (
     cromwell_status_to_status,
     CromwellMetadata,
@@ -28,12 +26,11 @@ from janis_assistant.engines.enginetypes import EngineType
 from janis_assistant.management.envvariables import EnvVariables
 from janis_assistant.utils import (
     ProcessLogger,
-    write_files_into_buffered_zip,
     find_free_port,
 )
 from janis_assistant.utils.dateutil import DateUtil
 from janis_assistant.utils.fileutil import tail
-from .cromwellconfiguration import CromwellConfiguration
+from .cromwellconfiguration import CromwellConfiguration, DatabaseTypeToUse
 from ..engine import Engine, TaskStatus
 
 CROMWELL_RELEASES = (
@@ -68,6 +65,7 @@ class Cromwell(Engine):
         config_path=None,
         execution_dir: str = None,
         polling_interval: Optional[int] = None,
+        db_type: DatabaseTypeToUse = None,
     ):
 
         super().__init__(
@@ -99,6 +97,8 @@ class Cromwell(Engine):
         # Last contacted is used to determine
         self.last_contacted = None
         self.timeout = 10  # minutes
+        self.db_type: Optional[DatabaseTypeToUse] = db_type
+        self.is_managing_cromwell = host is None
         if polling_interval is not None:
             polling_interval = int(polling_interval)
             if polling_interval < 3:
@@ -146,8 +146,7 @@ class Cromwell(Engine):
 
         try:
             r = request.urlopen(self.url_test())
-            r.raise_for_status()
-            return True
+            return r.code == 200
 
         except Exception as e:
             Logger.warn(f"Couldn't connect to Cromwell ({self.host}): {repr(e)}")
@@ -336,20 +335,23 @@ class Cromwell(Engine):
         self._process = None
 
     def stop_engine(self):
+
+        if not self.is_started:
+            return Logger.debug(
+                "Cromwell has already shut down, skipping shut down request"
+            )
+
         if self._logger:
             self._logger.terminate()
 
         self.should_stop = True
+
         if self._timer_thread:
             self._timer_thread.set()
 
-        if self._logfp:
-            self._logfp.flush()
-            os.fsync(self._logfp.fileno())
-            self._logfp.close()
-
         if not self.process_id:
-            Logger.warn("Could not find a cromwell process to end, SKIPPING")
+            self.is_started = False
+            Logger.info("Janis isn't managing Cromwell, skipping the shutdown")
             return
         Logger.info("Stopping cromwell")
         if self.process_id:
@@ -615,7 +617,7 @@ class Cromwell(Engine):
             newkey,
             WorkflowOutputModel(
                 id_=newkey,
-                original_path=fileloc,
+                original_path=None,
                 timestamp=DateUtil.now(),
                 new_path=None,
                 output_folder=None,
@@ -667,9 +669,10 @@ class Cromwell(Engine):
             shutil.copyfile(jc.cromwell.configpath, self.config_path)
 
         else:
-            self.config: CromwellConfiguration = jc.template.template.engine_config(
-                EngineType.cromwell, jc
-            ) or CromwellConfiguration()
+            self.config: CromwellConfiguration = (
+                jc.template.template.engine_config(EngineType.cromwell, jc)
+                or CromwellConfiguration()
+            )
             if not self.config.system:
                 self.config.system = CromwellConfiguration.System()
             self.config.system.cromwell_id = identifier
@@ -686,8 +689,10 @@ class Cromwell(Engine):
                     if not cnf.config.root:
                         cnf.config.root = self.execution_dir
             else:
-                self.config.backend = CromwellConfiguration.Backend.with_new_local_exec_dir(
-                    self.execution_dir
+                self.config.backend = (
+                    CromwellConfiguration.Backend.with_new_local_exec_dir(
+                        self.execution_dir
+                    )
                 )
 
     def raw_metadata(
@@ -742,9 +747,24 @@ class Cromwell(Engine):
 
         except (request.URLError, ConnectionResetError) as e:
             self.connectionerrorcount += 1
-            if (
+            minutes_not_able_to_contact_cromwell = (
                 datetime.now() - self.last_contacted
-            ).total_seconds() / 60 > self.timeout:
+            ).total_seconds() / 60
+            if minutes_not_able_to_contact_cromwell > self.timeout:
+                message = (
+                    f"Janis is receiving a ConnectionResetError when contacting the Cromwell instance "
+                    f"({self.host}) {self.connectionerrorcount} times, and has been unable to connect to "
+                    f"Cromwell for {minutes_not_able_to_contact_cromwell} minutes. "
+                )
+                if self.db_type and self.db_type == DatabaseTypeToUse.filebased:
+                    ja_config_url = "https://janis.readthedocs.io/en/latest/references/configuration.html#cromwell"
+                    message += (
+                        "We've seen this issue more frequently when Janis is configuring Cromwell to use the "
+                        "file-based database. We recommend configuring Janis to use a MySQL database through the "
+                        f"`--mysql` flag, visitng '{ja_config_url}', or raising an issue on GitHub ({GITHUB_URL}) "
+                        f"for more information."
+                    )
+                Logger.warn(message)
                 self.something_has_happened_to_cromwell(
                     "last_updated_threshold"
                 )  # idk, pick a number
