@@ -2,13 +2,16 @@ import os.path
 from enum import Enum
 from typing import Optional, List, Union
 import ruamel.yaml
+from janis_core.utils.logger import Logger
+
 from janis_assistant.utils import fully_qualify_filename
 
 from janis_assistant.engines.enginetypes import EngineType
-from janis_core.utils.logger import Logger
+from janis_assistant.engines.cromwell.cromwellconfiguration import DatabaseTypeToUse
 
 from janis_assistant.management.envvariables import EnvVariables, HashableEnum
 from janis_assistant.templates import from_template
+from janis_assistant.utils.callprogram import collect_output_from_command
 
 
 class NoAttributeErrors:
@@ -112,6 +115,7 @@ class JanisConfiguration(NoAttributeErrors):
         class Keys(HashableEnum):
             MaxCores = "max_cores"
             MaxRam = "max_ram"
+            MaxDuration = "max_duration"
 
         def __init__(self, d: dict, default: dict):
             d = d if d else {}
@@ -122,6 +126,9 @@ class JanisConfiguration(NoAttributeErrors):
             self.max_ram = JanisConfiguration.get_value_for_key(
                 d, self.Keys.MaxRam, default
             )
+            self.max_duration = JanisConfiguration.get_value_for_key(
+                d, self.Keys.MaxDuration, default
+            )
 
     class JanisConfigurationCromwell(NoAttributeErrors):
         class Keys(HashableEnum):
@@ -137,6 +144,7 @@ class JanisConfiguration(NoAttributeErrors):
             MySqlDbName = "mysql_dbname"
             UseManagedMySqlInstance = "should_manage_mysql"
             UseDatabase = "use_database"
+            GeneratedDbCredentials = "generated_db_credentials_from_script"
             AdditionalParameters = "additional_params"  # string or list of strings
             PollingInterval = "polling_interval"
 
@@ -185,6 +193,11 @@ class JanisConfiguration(NoAttributeErrors):
             self.use_database = JanisConfiguration.get_value_for_key(
                 d, self.Keys.UseDatabase, default
             )
+            self.generated_db_credentials_from_script = (
+                JanisConfiguration.get_value_for_key(
+                    d, self.Keys.GeneratedDbCredentials, default
+                )
+            )
 
             self.additional_params = JanisConfiguration.get_value_for_key(
                 d, self.Keys.AdditionalParameters, default
@@ -204,6 +217,7 @@ class JanisConfiguration(NoAttributeErrors):
                 mysql_config=existing_config,
                 skip_database=self.use_database is False,
                 run_managed_mysql_instance=self.mysql_instance,
+                generated_db_credentials_from_script=self.generated_db_credentials_from_script,
             )
 
     class JanisConfigurationRecipes(NoAttributeErrors):
@@ -412,9 +426,11 @@ class JanisConfiguration(NoAttributeErrors):
             default.get(JanisConfiguration.Keys.Cromwell),
         )
 
-        self.template: JanisConfiguration.JanisConfigurationTemplate = JanisConfiguration.JanisConfigurationTemplate(
-            d.get(JanisConfiguration.Keys.Template),
-            default.get(JanisConfiguration.Keys.Template),
+        self.template: JanisConfiguration.JanisConfigurationTemplate = (
+            JanisConfiguration.JanisConfigurationTemplate(
+                d.get(JanisConfiguration.Keys.Template),
+                default.get(JanisConfiguration.Keys.Template),
+            )
         )
 
         self.recipes = JanisConfiguration.JanisConfigurationRecipes(
@@ -527,12 +543,6 @@ class JanisConfiguration(NoAttributeErrors):
 
 
 class JanisDatabaseConfigurationHelper:
-    class DatabaseTypeToUse(Enum):
-        none = "none"
-        existing = "existing"
-        managed = "managed"
-        filebased = "filebased"
-
     class MySqlInstanceConfig:
         def __init__(self, url, username, password, dbname="cromwell"):
             self.url = url
@@ -545,25 +555,29 @@ class JanisDatabaseConfigurationHelper:
         mysql_config: MySqlInstanceConfig = None,
         run_managed_mysql_instance=None,
         skip_database=None,
+        generated_db_credentials_from_script=None,
     ):
         self.mysql_config = mysql_config
         self.should_manage_mysql = run_managed_mysql_instance
         self.skip_database = skip_database
+        self.generated_db_credentials_from_script = generated_db_credentials_from_script
 
     def which_db_to_use(self) -> DatabaseTypeToUse:
         if self.mysql_config is not None:
-            return self.DatabaseTypeToUse.existing
+            return DatabaseTypeToUse.existing
         elif self.skip_database:
-            return self.DatabaseTypeToUse.none
+            return DatabaseTypeToUse.none
         elif self.should_manage_mysql is True:
-            return self.DatabaseTypeToUse.managed
-        return self.DatabaseTypeToUse.filebased
+            return DatabaseTypeToUse.managed
+        elif self.generated_db_credentials_from_script is True:
+            return DatabaseTypeToUse.from_script
+        return DatabaseTypeToUse.filebased
 
     def get_config_for_existing_config(self):
         t = self.which_db_to_use()
-        if t != self.DatabaseTypeToUse.existing:
+        if t != DatabaseTypeToUse.existing:
             raise Exception(
-                f"Attempted to request database config for {self.DatabaseTypeToUse.existing.value} config, "
+                f"Attempted to request database config for {DatabaseTypeToUse.existing.value} config, "
                 f"but the database helper wants to use {t.value}"
             )
         from janis_assistant.engines.cromwell.cromwellconfiguration import (
@@ -581,9 +595,9 @@ class JanisDatabaseConfigurationHelper:
 
     def get_config_for_filebased_db(self, path):
         t = self.which_db_to_use()
-        if t != self.DatabaseTypeToUse.filebased:
+        if t != DatabaseTypeToUse.filebased:
             raise Exception(
-                f"Attempted to request database config for {self.DatabaseTypeToUse.filebased.value} config, "
+                f"Attempted to request database config for {DatabaseTypeToUse.filebased.value} config, "
                 f"but the database helper wants to use {t.value}"
             )
         from janis_assistant.engines.cromwell.cromwellconfiguration import (
@@ -594,9 +608,9 @@ class JanisDatabaseConfigurationHelper:
 
     def get_config_for_managed_mysql(self, url):
         t = self.which_db_to_use()
-        if t != self.DatabaseTypeToUse.managed:
+        if t != DatabaseTypeToUse.managed:
             raise Exception(
-                f"Attempted to request database config for {self.DatabaseTypeToUse.managed.value} "
+                f"Attempted to request database config for {DatabaseTypeToUse.managed.value} "
                 f"config, but the database helper wants to use {t.value}"
             )
 
@@ -608,6 +622,89 @@ class JanisDatabaseConfigurationHelper:
             username=None, password=None, url=url
         )
 
+    def get_config_for_template_supplied(self, output_dir: str):
+        try:
+            import subprocess, os, json
+            from janis_assistant.management.envvariables import EnvVariables
+            from janis_assistant.engines.cromwell.cromwellconfiguration import (
+                CromwellConfiguration,
+            )
+
+            file_path = os.getenv(EnvVariables.db_script_generator)
+            Logger.debug(
+                f"Found path '{EnvVariables.db_script_generator}' to generate database credentials"
+            )
+            if file_path is None:
+                raise Exception(
+                    f"Couldn't get database credentials as couldn't find value in env var '{EnvVariables.db_script_generator}'"
+                )
+            if not os.path.exists(file_path):
+                raise Exception(f"Couldn't locate script '{file_path}' to execute")
+
+            try:
+                val = collect_output_from_command(
+                    [file_path, output_dir], stderr=Logger.guess_log
+                )
+            except Exception as e:
+                Logger.critical(f"Failed to generate database credentials ({repr(e)})")
+                raise
+            d = json.loads(val)
+            Logger.debug(
+                "Received keys from database credentials script: " + ", ".join(d.keys())
+            )
+
+            keys = {"username", "password", "database", "host"}
+            missing_keys = {k for k in keys if k not in d}
+            if len(missing_keys) > 0:
+                raise Exception(
+                    "The script to generate database credentials was missing the keys: "
+                    + ", ".join(missing_keys)
+                )
+
+            return CromwellConfiguration.Database.mysql(
+                username=d["username"],
+                password=d["password"],
+                database=d["database"],
+                url=d["host"],
+            )
+        except Exception as e:
+            Logger.critical(
+                "Failed to get database configuration details from script: " + repr(e)
+            )
+            raise
+
+    def run_delete_database_script(self, execution_dir: str):
+        try:
+            import subprocess, os
+            from janis_assistant.management.envvariables import EnvVariables
+
+            file_path = os.getenv(EnvVariables.db_script_generator_cleanup)
+
+            if file_path is None:
+                raise Exception(
+                    f"Couldn't delete generated database credentials as couldn't find value in env var '{EnvVariables.db_script_generator_cleanup}'"
+                )
+            Logger.debug(
+                f"Found path '{EnvVariables.db_script_generator_cleanup}' to delete database credentials"
+            )
+            if not os.path.exists(file_path):
+                raise Exception(f"Couldn't locate script '{file_path}' to execute")
+
+            val = collect_output_from_command(
+                [file_path, execution_dir], stderr=Logger.guess_log
+            )
+            if val is not None and len(val) > 0:
+                Logger.info(
+                    f"Successfully deleted DB credentials and received message: {val}"
+                )
+            else:
+                Logger.info("Deleted credentials with rc=0")
+        except Exception as e:
+            Logger.warn(
+                f"Failed to delete database configuration details for execution directory '{execution_dir}': "
+                + repr(e)
+            )
+
 
 def stringify_dict_keys_or_return_value(d):
     if d is None:
