@@ -31,6 +31,7 @@ from janis_core import (
     Tool,
     WorkflowBase,
 )
+from janis_core.__meta__ import GITHUB_URL
 from janis_core.operators.operator import Operator
 from janis_core.translations import get_translator, CwlTranslator
 from janis_core.translations.translationbase import TranslatorBase
@@ -51,6 +52,7 @@ from janis_assistant.engines import (
 from janis_assistant.management.configuration import (
     JanisConfiguration,
     JanisDatabaseConfigurationHelper,
+    DatabaseTypeToUse,
 )
 from janis_assistant.management.filescheme import FileScheme, LocalFileScheme
 from janis_assistant.management.mysql import MySql
@@ -160,6 +162,7 @@ class WorkflowManager:
         watch=True,
         max_cores=None,
         max_memory=None,
+        max_duration=None,
         keep_intermediate_files=False,
         run_in_background=True,
         dbconfig=None,
@@ -176,7 +179,9 @@ class WorkflowManager:
         # output directory has been created
 
         tm = WorkflowManager(
-            submission_id=submission_id, execution_dir=execution_dir, engine=engine,
+            submission_id=submission_id,
+            execution_dir=execution_dir,
+            engine=engine,
         )
 
         tm.database.submissions.insert_or_update_many(
@@ -224,6 +229,7 @@ class WorkflowManager:
             additional_inputs=inputs_dict,
             max_cores=max_cores or jc.environment.max_cores,
             max_memory=max_memory or jc.environment.max_ram,
+            max_duration=max_duration or jc.environment.max_duration,
             allow_empty_container=allow_empty_container,
             container_override=container_override,
             check_files=check_files,
@@ -546,11 +552,9 @@ class WorkflowManager:
         self.copy_outputs_if_required()
 
         if status == TaskStatus.COMPLETED:
-            self.remove_exec_dir_if_required()
+            self.cleanup_execution()
 
-        self.engine.stop_engine()
-        if self.dbcontainer:
-            self.dbcontainer.stop()
+        self.stop_engine_and_db()
 
         rc = status.get_exit_code()
         if rc != 0:
@@ -593,7 +597,8 @@ class WorkflowManager:
             # in case anything relies on CD, we'll throw it into janis/execution
             os.chdir(self.get_path_for_component(self.WorkflowManagerPath.execution))
 
-            self.start_engine_if_required()
+            jc = JanisConfiguration.manager()
+            self.start_engine_if_required(jc)
 
             if os.path.exists(self.get_abort_semaphore_path()):
                 Logger.info("Detected please_abort request, aborting")
@@ -601,7 +606,7 @@ class WorkflowManager:
 
             if os.path.exists(self.get_pause_semaphore_path()):
                 Logger.info("Detected please_pause request, exiting")
-                return self.stop_computation()
+                return self.suspend_workflow()
 
             # check status and see if we can resume
             self.submit_workflow_if_required()
@@ -642,7 +647,7 @@ class WorkflowManager:
                             self.abort()
                             break
                         if os.path.exists(self.get_pause_semaphore_path()):
-                            self.stop_computation()
+                            self.suspend_workflow()
                             break
 
                     continue
@@ -669,9 +674,7 @@ class WorkflowManager:
                 )
                 self.database.commit()
 
-                self.engine.stop_engine()
-                if self.dbcontainer:
-                    self.dbcontainer.stop()
+                self.stop_engine_and_db()
 
                 self.database.close()
             except Exception as e:
@@ -690,6 +693,11 @@ class WorkflowManager:
             )
             sys.exit(rc)
 
+    def stop_engine_and_db(self):
+        self.engine.stop_engine()
+        if self.dbcontainer:
+            self.dbcontainer.stop()
+
     @staticmethod
     def mark_paused(execution_dir):
         try:
@@ -709,7 +717,7 @@ class WorkflowManager:
             Logger.critical("Couldn't mark paused: " + str(e))
             return False
 
-    def start_engine_if_required(self):
+    def start_engine_if_required(self, jc):
         # engine should be loaded from the DB
         engine = self.engine
 
@@ -722,28 +730,40 @@ class WorkflowManager:
             engine.start_engine()
             return
 
+        if not engine.is_managing_cromwell:
+            raise Exception(
+                f"Janis isn't managing Cromwell, and couldn't connect on '{engine.host}'"
+            )
+
         additional_cromwell_params = []
         if not engine.config:
             Logger.info("Skipping start database as Janis is not managing the config")
         else:
-            dbconfig: JanisDatabaseConfigurationHelper = self.database.submission_metadata.metadata.db_config
+            dbconfig: JanisDatabaseConfigurationHelper = (
+                self.database.submission_metadata.metadata.db_config
+            )
             dbtype = dbconfig.which_db_to_use()
-            if dbtype == dbconfig.DatabaseTypeToUse.existing:
+            engine.db_type = dbtype
+            if dbtype == DatabaseTypeToUse.existing:
                 engine.config.database = dbconfig.get_config_for_existing_config()
-            elif dbtype == dbconfig.DatabaseTypeToUse.filebased:
+            elif dbtype == DatabaseTypeToUse.filebased:
                 engine.config.database = dbconfig.get_config_for_filebased_db(
                     path=self.get_path_for_component(self.WorkflowManagerPath.database)
                     + "/cromwelldb"
                 )
-            elif dbtype == dbconfig.DatabaseTypeToUse.managed:
+            elif dbtype == DatabaseTypeToUse.managed:
                 cromwelldb_config = self.start_mysql_and_prepare_cromwell_config()
                 additional_cromwell_params.append(
                     "-Ddatabase.db.url=" + cromwelldb_config.db.url
                 )
                 engine.config.database = cromwelldb_config
+            elif dbtype == DatabaseTypeToUse.from_script:
+                engine.config.database = dbconfig.get_config_for_template_supplied(
+                    self.execution_dir
+                )
             else:
                 Logger.warn(
-                    "Skipping database config as '--no-database' option was provided."
+                    "Skipping database configuration as '--no-database' option was provided."
                 )
 
         engine_is_started = engine.start_engine(
@@ -829,6 +849,7 @@ class WorkflowManager:
         additional_inputs: dict,
         max_cores=None,
         max_memory=None,
+        max_duration=None,
         allow_empty_container=False,
         container_override: dict = None,
         check_files=True,
@@ -892,6 +913,7 @@ class WorkflowManager:
             additional_inputs=additional_inputs,
             max_cores=max_cores,
             max_mem=max_memory,
+            max_duration=max_duration,
             allow_empty_container=allow_empty_container,
             container_override=container_overrides,
         )
@@ -974,7 +996,7 @@ class WorkflowManager:
                     original_path=None,
                     new_path=None,
                     timestamp=None,
-                    output_name=output_names.get(o.id()),
+                    output_name=output_names.get(o.id(), True),
                     output_folder=output_folders.get(o.id()),
                     secondaries=secs,
                     extension=ext,
@@ -1239,18 +1261,20 @@ class WorkflowManager:
             output_name_prefix is False or output_name_prefix is None
         )
         has_original_path = (
-            isinstance(engine_output, WorkflowOutputModel)
-            and engine_output.original_path
+            iscopyable
+            and isinstance(engine_output, WorkflowOutputModel)
+            and isinstance(engine_output.value, str)
         )
         if using_original_filename and not has_original_path:
             using_original_filename = False
+            output_name_prefix = True
             Logger.warn(
-                "The output '{outputid}' requested original filename, but there wasn't a file to copy. "
+                f"The output '{outputid}' requested original filename, but there wasn't a file to copy. "
                 "Janis will use the output_id to construct the output name"
             )
 
         if using_original_filename:
-            outfn = os.path.basename(engine_output.original_path)
+            outfn = os.path.basename(engine_output.value)
         elif output_name_prefix is True:
             outfn = outputid
         else:
@@ -1272,11 +1296,21 @@ class WorkflowManager:
 
             ext = extension
             if ext is None and has_original_path:
-                ext = get_extension(engine_output.original_path)
+                ext = get_extension(engine_output.value)
+                if ext is not None:
+                    ext = "." + ext
             if ext:
                 # mfranklin: require user to correctly specific "." in extension
                 outfn += ext
 
+        if not isinstance(outfn, str):
+            Logger.warn(
+                f"An internal issue occurred when producing an output filename for '{outputid}', was expecting to use "
+                f"the original filename but this was not resolved correctly (resolved to: '{outfn}' [type: {type(outfn)}]). "
+                f"We'll instead use the output_id. Please raise an issue on GitHub ({GITHUB_URL}) for more information."
+            )
+            outfn = outputid
+
         if final_tags is None:
             final_tags = []
 
@@ -1290,20 +1324,36 @@ class WorkflowManager:
         newoutputfilepath = os.path.join(outdir, outfn)
 
         if isinstance(engine_output, WorkflowOutputModel):
-            original_filepath = engine_output.original_path
-            if original_filepath and iscopyable:
-                fs.cp_from(engine_output.original_path, newoutputfilepath, force=True)
-            elif engine_output.value:
-                if isinstance(fs, LocalFileScheme):
+            value = engine_output.value
+            if iscopyable:
+                if value is None:
+                    Logger.critical(
+                        f"Couldn't copy the output for '{outputid}', as the engine returned no path"
+                    )
+                else:
+                    fs.cp_from(value, newoutputfilepath, force=True)
+                    original_filepath = value
+            else:
+                if value is None:
+                    Logger.warn(f"The output '{outputid}' had no value")
+                elif isinstance(fs, LocalFileScheme):
                     # Write engine_output to outpath
                     with open(newoutputfilepath, "w+") as outfile:
                         outfile.write(str(engine_output.value))
+                else:
+                    Logger.warn(
+                        f"Skipping writing the output value for '{outputid}' as Janis doesn't support writing values not on the local filesystem"
+                    )
         else:
             original_filepath = engine_output
             if isinstance(fs, LocalFileScheme):
                 # Write engine_output to outpath
                 with open(newoutputfilepath, "w+") as outfile:
                     outfile.write(str(engine_output))
+            else:
+                Logger.critical(
+                    f"Can't write output '{outputid}' to non-local filesystem '{fs.id()}'"
+                )
 
         for sec in secondaries or []:
             frompath = apply_secondary_file_format_to_filename(original_filepath, sec)
@@ -1313,7 +1363,7 @@ class WorkflowManager:
 
         return [original_filepath, newoutputfilepath]
 
-    def remove_exec_dir_if_required(self):
+    def cleanup_execution(self):
         status = self.database.submission_metadata.metadata.status
 
         keep_intermediate = (
@@ -1330,6 +1380,13 @@ class WorkflowManager:
                 self.filescheme.rm_dir(execdir)
                 self.database.progressDB.set(ProgressKeys.cleanedUp)
 
+            dbconfig: JanisDatabaseConfigurationHelper = (
+                self.database.submission_metadata.metadata.db_config
+            )
+            dbtype = dbconfig.which_db_to_use()
+            if dbtype == DatabaseTypeToUse.from_script:
+                dbconfig.run_delete_database_script(self.execution_dir)
+
     def log_dbtaskinfo(self):
         import tabulate
 
@@ -1447,6 +1504,7 @@ class WorkflowManager:
                 ),
                 "abort",
             )
+            Logger.info(f"Aborting task '{wid}' by writing to '{path}'")
             with open(path, "w+") as f:
                 f.write(f"Requesting abort {DateUtil.now()}")
             return True
@@ -1470,24 +1528,20 @@ class WorkflowManager:
         except Exception as e:
             Logger.critical("Couldn't abort task from engine: " + str(e))
         try:
-            engine.stop_engine()
-            if self.dbcontainer:
-                self.dbcontainer.stop()
+            self.stop_engine_and_db()
         except Exception as e:
             Logger.critical("Couldn't stop engine: " + str(e))
 
         return status
 
-    def stop_computation(self):
+    def suspend_workflow(self):
         try:
             self.set_status(TaskStatus.SUSPENDED)
             # reset pause flag
             self.database.commit()
             self.database.submission_metadata.save_changes()
 
-            self.engine.stop_engine()
-            if self.dbcontainer:
-                self.dbcontainer.stop()
+            self.stop_engine_and_db()
 
             self.database.close()
 
